.module Menu_Manager

.include "cpctelera.h.s"
.include "includes.s"
.include "game_manager.h.s"
.include "sys/render.h.s"
.include "sys/input.h.s"
.include "sprites/menu.h.s"
.include "menu.h.s"

string_play: .asciz "PLAY"
string_credits: .asciz "CREDITS"
menu_focus: .db 0
key_pressed: .db 0

;; =================== INIT DE MENU INICIAL =================== ;
;; MAS DETALLES
;;  Descomprime la imagen y por defecto el focus esta en "PLAY"
;; ============================================================ ;
menu_init::
    ;; Inicializa el render para tener el borde, el modo de video y la paleta actualizada
    call sys_render_init

    ;; Direccion final a la que se va a copiar la imagen de fondo
    ld de, #0xFFFF
    ;; Direccion final de la imagen de fondo comprimida
    ld hl, #_menu_end
    ;; Llamada al descompresor
    call cpct_zx7b_decrunch_s_asm

    ;; Por defecto, imprimimos el menu con el focus en PLAY
    call focus_play
    xor a
    ld (menu_focus), a
    ret
;; ============================================================ ;



;; ====================== UPDATE DEL MENU ===================== ;
;; MAS DETALLES
;;  Escanea el teclado, cambia el focus
;; SALIDA
;;  "A" con el indice de la opcion cuando se pulse ENTER
;; ============================================================ ;
menu_update::
    call cpct_waitVSYNC_asm
    call cpct_scanKeyboard_f_asm
    
    call cpct_isAnyKeyPressed_f_asm     ;; DEVUELVE A = "0" CUANDO NO HAY TECLAS PULSADAS
    or #0
    jr nz, key_not_pressed
    ld a, #0
    ld (key_pressed), a

    key_not_pressed:
    ld a, (key_pressed)
    or a
    jr nz, Q_NotPressed

    ;; IF @( KEY IS STILL PRESSED )
    ;; {
        ld hl, #Key_A
        call cpct_isKeyPressed_asm
        jr z, A_NotPressed
        ;; IF @( "A" IS PRESSED )
        ;; {
            ld a, (menu_focus)
            inc a
            call update_focus
        ;; }
        A_NotPressed:

        ld hl, #Key_Q
        call cpct_isKeyPressed_asm
        jr z, Q_NotPressed
        ;; IF @( "Q" IS PRESSED )
        ;; {
            ld a, (menu_focus)
            dec a
            call update_focus
        ;; }
        Q_NotPressed:
    ;; }

    ld hl, #Key_Return
    call cpct_isKeyPressed_asm
    jr z, menu_update
    ld a, (menu_focus)
    inc a
    ret

;; ============================================================ ;



;; ==================== ACTUALIZA EL FOCUS ==================== ;
;; MAS DETALLES
;;  Dependiendo de la variable "FOCUS", cambia el focus entre una palabra u otra
;; ENTRADA
;;  "A" = Indice de la opcion del menu
;; ============================================================ ;
update_focus:
    ;; Si el numero de opcion es negativo -> ha pulsado arriba en la primera opcion
    ;; Solucion = Hacer focus en la ultima opcion del menu
    bit 7, a
    jr z, indice_no_negativo
    ;; IF @( "A" MENOR QUE 0 )
    ;; {
        ld a, #N_Menu_Opciones-1        ;; (N_Menu_Opciones - 1) = Ultima opcion
    ;; }

    indice_no_negativo:
    ;; Si el numero de opcion es mayor que el num de opciones -> ha pulsado abajo en la ultima opcion
    ;; Solucion = Hacer focus en la primera opcion del menu
    ;; ELSE IF @( "A" MAYOR QUE N_MENU_OPCIONES )
    ;; {
        cp #N_Menu_Opciones         ;; Le resto el num de opciones
        jr nz, indice_adecuado      ;; En caso de que el resultado sea 0
        ld a, #0                    ;; Poner el focus en 0
    ;; }


    ;; Una vez adaptado el indice a unos parametros correctos
    indice_adecuado:
    ld (menu_focus), a
    cp #0
    jr z, etiq_focus_play
    ;; IF @( FOCUS != PLAY )
    ;; {
        call focus_credits      ;; Para menu_focus = 1 -> Focus creditos
        jr end_focus
    ;; }

    ;; ELSE 
    ;; {
        etiq_focus_play:
        call focus_play         ;; Para menu_focus = 0 -> Focus jugar
    ;; }


    end_focus:
    ld a, #1                    ;; Para solo leer una lectura de teclado
    ld (key_pressed), a         ;; Key_Pressed se pone a 1 hasta que no haya teclas pulsadas
    ret

;; ============================================================ ;



;; ==================== ACTUALIZA EL FOCUS ==================== ;
;; MAS DETALLES
;;  Dibuja las cadenas del mapa, con el focus en PLAY
;; ============================================================ ;
focus_play:
    ld hl, #0x0006
    call dibujar_cadena_play
    ld hl, #0x0005
    call dibujar_cadena_credits
    ret


;; ==================== ACTUALIZA EL FOCUS ==================== ;
;; MAS DETALLES
;;  Dibuja las cadenas del mapa, con el focus en CREDITS
;; ============================================================ ;
focus_credits:
    ld hl, #0x0005
    call dibujar_cadena_play
    ld hl, #0x0006
    call dibujar_cadena_credits
    ret


;; ==================== ACTUALIZA EL FOCUS ==================== ;
;; MAS DETALLES
;;  Dibuja "PLAY" con el color que le hayan pasado
;; ============================================================ ;
dibujar_cadena_play:
    call cpct_setDrawCharM0_asm

    ld iy, #string_play
    ld de, #CPCT_VMEM_START_ASM
    ld c, #45
    ld b, #67
    call cpct_getScreenPtr_asm
    call cpct_drawStringM0_asm
    ret


;; ==================== ACTUALIZA EL FOCUS ==================== ;
;; MAS DETALLES
;;  Dibuja "CREDITS" con el color que le hayan pasado
;; ============================================================ ;
dibujar_cadena_credits:
    call cpct_setDrawCharM0_asm
    
    ld iy, #string_credits
    ld de, #CPCT_VMEM_START_ASM
    ld c, #45
    ld b, #82
    call cpct_getScreenPtr_asm
    call cpct_drawStringM0_asm
    ret