.include "cpctelera.h.s"
.include "render.h.s"
.include "man/game_manager.h.s"
.include "man/manager_entity.h.s"
.include "man/level.h.s"
.include "camera.h.s"
.include "man/sfx.h.s"

_back_buffer:: .dw 0x8000
extra_tile_loop:: .db 0
update_UI:: .db 0
string_score: .asciz "SCORE"
color_fondo: .db 0


;; ==================== INIT DE SYS RENDER ==================== ;
sys_render_init::
  ;; MODO DE VIDEO = 0
  ld c, #0
  call cpct_setVideoMode_asm
   
  ;; ASIGNAR LA PALETA, TAMANYO 16
  ld hl, #_g_palette
  ld de, #16
  call cpct_setPalette_asm
  cpctm_setBorder_asm #HW_BLACK

  ld a, #2
  ld (extra_tile_loop), a
  ld a, #2
  ld (update_UI), a

  ret

;; ============================================================ ;



;; =================== RENDERIZA UNA ENTIDAD ================== ;
;; MAS DETALLES
;;  Borra la antigua posicion y dibuja la entidad en su (x,y)
;; ENTRADA
;;  IX = Posicion en memoria de la entidad a renderizar
;; ============================================================ ;
sys_render_one_entity::
   ;; Para volver a llamar al render
   ld a, (ix)
   and #TYPE_DESTROY
   jr nz, endif_no_renderizar

    ;; Si se encuentra fuera de la camara, no se renderiza la entidad
    ld a, e_sx(ix)
    cp #70
    ret nc

    ;; =========== DIBUJAR POS ACTUAL ============ ;;
     ;; "DE" = Direccion inicio de pantalla -> C000
     ld de, (#_back_buffer)
     ;; "C" = Posicion X de la entidad
     ld c, a
     ;; "B" = Posicion Y de la entidad
     ld b, e_sy(ix)
     ;; DE = fijo , C = Posicion_X , B = Posicion_Y
     ;; En "HL" se guarda el puntero
     call cpct_getScreenPtr_asm

     
     ;;(2B BC) psprite	Source Sprite Pointer
     ;;(2B DE) pvideomem	Destination video memory pointer
     ;;(1B IXL) width	Sprite Width in bytes (>0) (Beware, not in pixels!)
     ;;(1B IXH) height	Sprite Height in bytes (>0)
     ;;(2B HL) pmasktable0	Pointer to an Aligned Mask Table for transparencies with palette index 0
     ex de, hl
     ld b, e_sprite_h(ix)
     ld c, e_sprite_l(ix)
     ld a, e_w(ix)
     push ix
     ld__ixl_a
     push ix
     pop hl
     pop ix
     ld a, e_h(ix)
     push hl
     pop ix
     ld__ixh_a
     ld hl, #0x0100
     call cpct_drawSpriteMaskedAlignedTable_asm

    ;; =========================================== ;;
   endif_no_renderizar:
  ret

;; ============================================================ ;



;; ==================== UPDATE DEL RENDER ===================== ;
;; MAS DETALLES
;;  Llama a todos los FOR-ALL y les pasa render_one_entity en HL
;; ============================================================ ;
sys_render_update::

  ld a, (extra_tile_loop)       ;; Si extra > 0
  cp #0                         ;; Render tilemap completo
  call nz, sys_render_tilemap

  ld a, (man_level_map_camera + cam_state)
  or a
  call z, sys_render_tilemap_action_rect
  call nz, sys_render_tilemap

  ld a, (color_fondo)
  cp #0
    jr z, color_borde_normal
        cpctm_setBorder_asm #HW_RED
        jr end_color_borde
    color_borde_normal:
    cpctm_setBorder_asm #HW_BLACK

    end_color_borde:



  ld b, #COMP_RENDER
  ;; HL = direccion de memoria "SYS_RENDER_ONE_ENTITY"
  ld hl, #sys_render_one_entity
  ;; Llamo a "MAN_ENTITY_FORALL"
  ;; Que hace un bucle por cada entidad
  call entity_man_forall_but_player

  ;; HL = direccion de memoria "SYS_RENDER_ONE_ENTITY"
  ld hl, #sys_render_one_entity
  ;; Llamo a "MAN_ENTITY_FORALL"
  ;; Que hace un bucle por cada entidad
  call entity_man_forjust_player

  ld hl, #sys_render_one_entity
  call man_sfx_forall

  ld a, (update_UI)
  cp #0
  call nz, sys_render_UI

  call change_screen
  ret

;; ============================================================ ;



;; ============== DIBUJA LA INTERFAZ DE USUARIO =============== ;
;; MAS DETALLES
;;  Dibuja un rectangulo de color negro en la parte baja de la pantalla
;;  Solo ocurre la primera vez que inicia el juego
;; ============================================================ ;
sys_render_UI::

  ld a, (update_UI)
  sub #1
  ld (update_UI), a

  ;; ============ DIBUJAR RECTANGULO NEGRO ============ ;
    ld de, (_back_buffer)
    ld bc, #0xA000                ;; Posicion del primer rectangulo
    call cpct_getScreenPtr_asm    ;; "HL" = Puntero en la pantalla
    ex de, hl
    ld a, #12                     ;; Fondo color negro
    ld c, #40                     ;; Ancho = 40
    ld b, #40                     ;; Alto  = 40
    call cpct_drawSolidBox_asm    ;; Dibujar rectangulo

    ld de, (_back_buffer)
    ld bc, #0xA028                ;; Posicion del primer rectangulo (40 pos mas)
    call cpct_getScreenPtr_asm
    ex de, hl
    ld a, #12                     ;; Fondo color negro
    ld c, #40                     ;; Ancho = 40
    ld b, #40                     ;; Alto  = 40
    call cpct_drawSolidBox_asm    ;; Dibujar rectangulo
  ;; ================================================== ;


  ;; =============== DIBUJAR PUNTUACION =============== ;
   call sys_render_score
  ;; ================================================== ;


  ;; ============ DIBUJAR PALABRA "COMBO" ============= ;
    ld a, (#entities + e_attacking)
    cp #2
    jr nc, if_combo_ok

    ld de, (_back_buffer)
    ld bc, #0xA832                ;; Posicion donde va la palabra
    call cpct_getScreenPtr_asm
    
    ex de, hl
    ld hl, #_SP_ComboOFF          ;; Sprite de la palabra "COMBO" apagada
    ld bc, #0x0F1B                ;; Ancho y alto del sprite
    call cpct_drawSprite_asm      ;; Dibujar sprite (SIN MASCARA!)
    jr endif_combo_ok
    
    if_combo_ok:
    call sys_render_combo_ok

    endif_combo_ok:
  ;; ================================================== ;


  ;; ============ DIBUJAR CORAZONES PLAYER ============ ;
    ld hl, #_SP_LifeIcon
    push hl
    ld bc, #0xAA00                ;; Posicion donde empiezan a dibujarse los corazones
    push bc                       ;; Guardo esta posicion en la pila

    ld a, (#entities + e_vidas)   ;; Guardo en el registro "A" las vidas del jugador
    ld__iyl #1
    
    ;; Bucle que renderiza los corazones dependiendo del numero de vidas
    bucle_render_vidas:
      pop bc
      ld l, c                       ;; /
      ld de, #0x0005                ;; | En "BC" esta la posicion del ultimo corazon dibujado
      add hl, de                    ;; | Y le sumbo 10d que es el ancho del sprite del corazon
      ld c, l                       ;; \
      pop hl

      cp #0                         ;; Compruebo si ya he dibujado todas las vidas
      jr z, fin_corazones_rojos
      ;; IF @( A > 0 ) -> IMPRIMIR OTRO CORAZON
      ;; {
        ld de, (_back_buffer)
        push bc                     ;; / Meto algunos registros en la pila
        push af                     ;; \ Porque getScreenPtr me destroza los valores
        push hl
        call cpct_getScreenPtr_asm

        ex de, hl                   ;; "DE" = Puntero a la posicion donde se va a dibujar
        pop hl
        push hl
        ld bc, #0x0905              ;; Dimensiones del sprite
        call cpct_drawSprite_asm    ;; Dibujar el sprite

        pop hl
        pop af                      ;; / Recupero el valor de "A"
        dec a                       ;; \ Y decremento uno (al registro, no a las vidas del jugador)
        pop bc
        push hl
        push bc

        jr bucle_render_vidas       ;; Dibujar siguiente corazon
    ;; }

  ;; ================================================== ;

  fin_corazones_rojos:
    ld__a_iyl
    or a
    ret z

      ld l, c                       ;; /
      ld de, #-0x0005                ;; | En "BC" esta la posicion del ultimo corazon dibujado
      add hl, de                    ;; | Y le sumbo 10d que es el ancho del sprite del corazon
      ld c, l                       ;; \

      ld a, (vidas_extra_actuales)
      ld hl, #_SP_LifeIcon_extra
      push hl
      push bc
      ld__iyl #0
      jr bucle_render_vidas

  ret
;; ============================================================ ;



;; ==================== DIBUJAR EL TILEMAP ==================== ;
;; MAS DETALLES
;;  Dibuja el tilemap
;; ============================================================ ;
sys_render_tilemap::
  ;; "BC" = Tamanyo de la ventana (20 x 20)
  ld bc, #0x1414
  ;; "DE" = Ancho total del tilemap
  ld de, #0x3C
  ;; Direccion a los tiles que se usaran en el dibujado
  ld hl, #_tiles_000
  call cpct_etm_setDrawTilemap4x8_ag_asm
  
  ;; Apuntar a primer tile que se dibuja en la esquina superior izquierda
  ld de, (man_level_map_camera)
  ;; Lugar de la memoria donde se dibuja (doble buffer)
  ld hl, (#_back_buffer)
  call cpct_etm_drawTilemap4x8_ag_asm

  ;; Si el estado vale 1, la siguiente iteracion no se dibujara
  ld a, (man_level_map_camera + cam_state)
  cp #1
  jr nz, posible_ultima_vuelta
  ;; IF @( LA SIGUIENTE ITERACION == LA ULTIMA )
  ;; {
    efectivamente_es_el_ultimo:
    ld a, #2
    ld (extra_tile_loop), a
    jr endif_posible_vuelta
  ;; }

  posible_ultima_vuelta:
  cp #-1
  jr z, efectivamente_es_el_ultimo
  ld a, (extra_tile_loop)
  cp #0
  jr z, endif_posible_vuelta
    dec a
    ld (extra_tile_loop), a

  endif_posible_vuelta:
  ret

;; ============================================================ ;



;; ==================== DIBUJAR EL TILEMAP2 ==================== ;
;; MAS DETALLES
;;  Dibuja el tilemap
;; ============================================================ ;
sys_render_tilemap_action_rect::
  ld bc, #0x0914
  ld de, #0x3C
  ld hl, #_tiles_000
  call cpct_etm_setDrawTilemap4x8_ag_asm
  
  ld hl, (man_level_map_camera)
  ld de, #360           ;; 60 x 6
  add hl, de
  ex de, hl
  ld hl, (#_back_buffer)
  ld bc, #0x01E0        ;; Posicion de lo que se ve
  add hl, bc
  call cpct_etm_drawTilemap4x8_ag_asm
  ret

;; ============================================================ ;



;; ================= DIBUJAR COMBO ILUMINADO ================== ;
sys_render_combo_ok::
  ld de, #0xC000
  ld bc, #0xA832                ;; Posicion donde va la palabra
  call cpct_getScreenPtr_asm
  ex de, hl
  ld hl, #_SP_ComboON
  ld bc, #0x101E                ;; Ancho y alto del sprite
  call cpct_drawSprite_asm

  ld de, #0x8000
  ld bc, #0xA832                ;; Posicion donde va la palabra
  call cpct_getScreenPtr_asm
  ex de, hl
  ld hl, #_SP_ComboON
  ld bc, #0x101E                ;; Ancho y alto del sprite
  call cpct_drawSprite_asm

  ret

;; ============================================================ ;



;; ================== ETIQUETAS DOBLE BUFFER ================== ;
;; MAS DETALLES
;;  Cambiar pantalla, alterna entre 8000 y C000 con codigo automodificalbe
;; ============================================================ ;
change_screen:
  f_change_screen = . + 1
  jp change_screen_to_8000


;; ================== ETIQUETAS DOBLE BUFFER ================== ;
;; MAS DETALLES
;;  Cambia la pantalla a la posicion 8000 y cambia el back_buffer
;; ============================================================ ;
change_screen_to_8000:
  ld bc, #0x200C
  call cpct_setCRTCReg_asm

  ld hl, #0xC000
  ld (_back_buffer), hl

  ld hl, #change_screen_to_C000
  ld (f_change_screen), hl

  ret


;; ================== ETIQUETAS DOBLE BUFFER ================== ;
;; MAS DETALLES
;;  Cambia la pantalla a la posicion C000 y cambia el back_buffer
;; ============================================================ ;
change_screen_to_C000:
  ld bc, #0x300C
  call cpct_setCRTCReg_asm

  ld hl, #0x8000
  ld (_back_buffer), hl

  ld hl, #change_screen_to_8000
  ld (f_change_screen), hl
  ret
;; ============================================================ ;


;;
sys_render_UI_both::
  ld a, #2
  ld (update_UI), a
  ret
;;


;; ==================== DIBUJAR PUNTUACION ==================== ;
sys_render_score::
  ld hl, #0x0204                ;; Letras blancas sobre fondo negro
  call cpct_setDrawCharM0_asm   ;; El score se dibujara con este estilo

  ld de, (_back_buffer)
  ld bc, #0xB221                ;; Posicion donde va la palabra
  call cpct_getScreenPtr_asm
  
  ex de, hl
  ld hl, #_SP_Score             ;; Sprite de la palabra "COMBO" apagada
  ld bc, #0x090D                ;; Ancho y alto del sprite
  call cpct_drawSprite_asm      ;; Dibujar sprite (SIN MASCARA!)

  ld iy, #0xA822
  ld d, #0
  ;; PRIMEROS DOS DIGITOS DE PUNTUACION
  ld ix, #puntuacion
  ld a, 1(ix)
  or a
  jr z, solo_dos_digitos2
  ;; LO NORMALIZO A DECIMAL
  daa
  call centrar_puntos
  jr solo_dos_digitos

  solo_dos_digitos:
  ld d, #1

  solo_dos_digitos2:
  ;; "IY" = LA POSICION DONDE DIBUJAR LOS DOS DIGITOS
  ld bc, #4
  add iy, bc
  ;; PRIMEROS DOS DIGITOS DE PUNTUACION
  ld ix, #puntuacion
  ld a, 0(ix)
  ;; LO NORMALIZO A DECIMAL
  call centrar_puntos
  ret
  




;;=== PARA CENTRAR EL TEXTO, ES UN LIO PERO QUEDA BIEN === ;
centrar_puntos:
  push af

  ld a, d
  cp #1
  jr z, zero_necesario

  pop af
  push af
  ;; "BC" = -10   -> Para comprobar si solo tengo que dibujar un digito
  ld c, #-10
  add a, c
  jr nc, puntuacion_un_digito
  
  ;; IF @( PUNTUACION >= 10 ENTONCES DIBUJAR 2 DIGITOS )
  ;; {
    zero_necesario:
      ld de, (_back_buffer)
      ;;ld bc, #0xA823                ;; Posicion donde va la palabra
      ld bc, #-2
      add iy, bc
      push iy
      pop bc
      call cpct_getScreenPtr_asm

      pop af
      push af
      rra
      rra
      rra
      rra
      and #0x0F
      add #48
      ld e, a
      call cpct_drawCharM0_asm


      ld bc, #4
      add iy, bc
      jr puntuacion_un_digito
      ;;ld bc, #0xA827                ;; Posicion donde va la palabra
  ;; }


  ;; ELSE IF @( PUNTUACION < 10 ENTONCES DIBUJAR 1 DIGITO )
  ;; {
  puntuacion_un_digito:

    ld de, (_back_buffer)
    push iy
    pop bc
    call cpct_getScreenPtr_asm

    pop af
    and #0x0F
    add #48
    ld e, a
    call cpct_drawCharM0_asm
  ;; }
      ld bc, #2
      add iy, bc
  ret

;; ============================================================ ;
